<?php
/**
 * Copyright © 2015 FiloBlu . All rights reserved.
 */

namespace FiloBlu\Flow\Helper;

use FiloBlu\Flow\Api\Data\SpecialActionPoolInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Module\Manager;
use Magento\Framework\ObjectManagerInterface;

/**
 * Class Data
 * @package FiloBlu\Flow\Helper
 */
class Data extends AbstractHelper
{
    /**
     * @var AdapterInterface
     */
    protected $_connection;

    /**
     * @var
     */
    protected $_storesByLocale;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var SpecialActionPoolInterface
     */
    protected $specialActionPool;

    /**
     * @var Manager
     */
    protected $moduleManager;

    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;

    /**
     * Data constructor.
     * @param Context $context
     * @param ResourceConnection $_connection
     * @param StoreManagerInterface $storeManager
     * @param SpecialActionPoolInterface $specialActionPool
     * @param Manager $moduleManager
     * @param ObjectManagerInterface $objectManager
     */
    public function __construct(
        Context $context,
        ResourceConnection $_connection,
        StoreManagerInterface $storeManager,
        SpecialActionPoolInterface $specialActionPool,
        Manager $moduleManager,
        ObjectManagerInterface $objectManager
    ) {
        $this->_connection = $_connection->getConnection();
        $this->storeManager = $storeManager;
        $this->specialActionPool = $specialActionPool;
        $this->moduleManager = $moduleManager;
        $this->objectManager = $objectManager;
        parent::__construct($context);
    }

    /**
     * @return mixed
     */
    public function getModels()
    {
        $channels_in = (Array)$this->scopeConfig->getValue('processor/channels/in');
        $channels_out = (Array)$this->scopeConfig->getValue('processor/channels/out');

        $classes['select'] = '';

        foreach ($channels_in as $key => $channel) {
            $classes[$channel] = $key;
        }

        foreach ($channels_out as $key => $channel) {
            $classes[$channel] = $key;
        }

        return $classes;
    }

    /**
     * @param $channel_input
     * @return int|string|void
     */
    public function getChannelModels($channel_input)
    {
        $channels_in = $this->scopeConfig->getValue('processor/channels/in');

        foreach ($channels_in as $key => $channel) {
            if ($channel === $channel_input) {
                return $key;
            }
        }
    }

    /**
     * @param $idInboundflow
     * @return array
     */
    public function getChannelByInbound($idInboundflow)
    {
        if ($idInboundflow > 0) {
            $result = $this->_connection->fetchRow(
                'SELECT * FROM flow_inboundflow WHERE id = ? LIMIT 1',
                [$idInboundflow]
            );

            if (count($result) > 0) {
                return $result;
            }
        }

        return [];
    }

    /**
     * @param $text
     * @return string
     */
    public function removeTextSpaces($text)
    {
        $result = preg_replace("/\s/u", '', $text);
        $result = preg_replace('/( )+/u', '', $result);
        return trim($result);
    }

    /**
     * @return array
     */
    public function getAllStoreIds()
    {
        $storeIds = [];

        foreach ($this->storeManager->getStores($withDefault = true) as $store) {
            $storeIds[$store->getCode()] = $store->getId();
        }

        return $storeIds;
    }

    /**
     * @return array
     */
    public function getStoreInWebsiteId()
    {
        $storeIds = [];

        $listStores = $this->storeManager->getStores($withDefault = true);
        $listWebsites = $this->storeManager->getWebsites($withDefault = true);

        foreach ($listWebsites as $website) {
            foreach ($listStores as $store) {
                if ($website->getId() === $store->getWebsiteId()) {
                    $storeIds[$website->getCode()] = [
                        'store_id' => $store->getId()
                    ];
                    break;
                }
            }
        }

        return $storeIds;
    }

    /**
     * TODO : to be tested
     *
     * @return array
     */
    public function getAllStoresCodeByLocale()
    {
        $stores = $this->storeManager->getStores($withDefault = false);

        //Locale code
        $storesByLocale = [];

        //Try to get list of locale for all stores;
        foreach ($stores as $store) {
            $localeCode = $this->scopeConfig->getValue(
                'general/locale/code',
                ScopeInterface::SCOPE_STORE,
                $store->getId()
            );
            $localeCodeFirstPart = substr($localeCode, 0, 2);
            $localeCodeSecondPart = strtolower(substr($localeCode, 3, 2));
            $storeCode = $store->getCode();
            $storesByLocale[$localeCodeFirstPart][] = $storeCode;

            /* To cover more cases during the import, check if the first two letters of the ISO2 code are different from the last two;
               in this case I also add the latter as a language code. */
            if($localeCodeFirstPart != $localeCodeSecondPart)
            {
                $storesByLocale[$localeCodeSecondPart][] = $storeCode;
            }
        }

        return $storesByLocale;
    }

    /**
     * @param $locale
     * @return array|mixed
     */
    public function getStoresCodeByLocale($locale)
    {
        if ($locale === '') {
            return [];
        }

        if (count($this->_storesByLocale) === 0) {
            $stores = $this->storeManager->getStores($withDefault = false);

            //Locale code
            $this->_storesByLocale = [];

            //Try to get list of locale for all stores;
            foreach ($stores as $store) {
                $localeCode = $this->scopeConfig->getValue(
                    'general/locale/code',
                    ScopeInterface::SCOPE_STORE,
                    $store->getId()
                );
                $localeCode = substr($localeCode, 0, 2);

                if (!isset($this->_storesByLocale[$localeCode])) {
                    $this->_storesByLocale[$localeCode] = [];
                }
                $storeCode = $store->getCode();
                $this->_storesByLocale[$localeCode][] = $storeCode;
            }
        }

        if (isset($this->_storesByLocale[$locale])) {
            return $this->_storesByLocale[$locale];
        }
        return [];
    }

    /**
     * @return mixed
     */
    public function getSkeleton()
    {
        return str_replace(
            ["\n", ' '],
            '',
            '{"field":{"from":"","to":"","type":"text","validators":[],"filters":[],"stores":"","locale":"","actions":[]} }'
        );
    }

    /**
     * @return array
     */
    public function getSpecialActionsPriorities()
    {
        return $this->specialActionPool->getActions();
    }

    /**
     * @return array
     */
    public function getListLanguageForAllStores()
    {
        //Locale code
        $allLocale = [];

        $stores = $this->storeManager->getStores($withDefault = false);
        //Try to get list of locale for all stores;
        foreach ($stores as $store) {
            //$locale[] = $this->scopeConfig->getValue('general/locale/code', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store->getStoreId());
            $locale = $this->scopeConfig->getValue(
                'general/locale/code',
                ScopeInterface::SCOPE_STORE,
                $store->getId()
            );
            $langCode = explode('_', $locale);
            $lang = trim($langCode[0]);
            if (!isset($allLocale[$lang])) {
                $allLocale[$lang] = [];
            }
            $allLocale[$lang][] = $store->getId();
        }
        return $allLocale;
    }

    /**
     * @return array
     */
    public function getSources(){

        $sources = [];

        if ($this->moduleManager->isEnabled('Magento_Inventory')) {

            $inventorySourceCollection = $this->objectManager->create('\Magento\Inventory\Model\ResourceModel\Source\CollectionFactory');

            foreach ($inventorySourceCollection->create()->getItems() as $source) {
                $sourceCode = $source->getSourceCode();
                $sourceName = $source->getName();
                $sources[$sourceCode] = $sourceName;
            }
        }
        else{
            $sources['default'] = 'Default';
        }

        return $sources;
    }
}
